// This is free software.
// You may redistribute it by any means.
//
// No warrantee of any kind is included (use at your own risk).
//
// I can be reached for bug reports, bug fixes, enhancements, requests, 
//	flames, etc. at:
//    MarkI@Asymetrix.com                
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
//CComCollection is a template collection class the requires the following:
//
//T:	The type of Interfaces that you're collecting. 
//			ie. IBars collects IBar so T would be IBar
//
//R:	The actual collection class.
//			ie. CBars
//
//Q:	The interface of the collection class.
//			ie. IBars
//
//Pointers to your: CLSID, IID, and LIBID	
/////////////////////////////////////////////////////////////////////////////

#ifndef __COMCOLLECTION_H__
#define __COMCOLLECTION_H__

#pragma warning(disable:4530)	//Template memory warning

#include <vector>
using namespace std;

template <class T, class R, class Q, const CLSID* pclsid, const IID* piid, const GUID* plibid> 
class ATL_NO_VTABLE CComCollection : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<R, pclsid>,
	public IDispatchImpl<Q, piid, plibid>,
	public IEnumVARIANT
{

	typedef vector< CComPtr<T> > COLLECTION_VECTOR;
	typedef COLLECTION_VECTOR::iterator COLLECTION_ITERATOR;

public:
	CComCollection()
	{
		m_ColVector.clear();
		m_ColIterator = m_ColVector.begin();
	}

	~CComCollection()
	{
		m_ColVector.clear();
	}

	//Standard Collection methods
	//
	STDMETHOD(Add)(T * lp)
	{
		m_ColVector.push_back(lp);

		return S_OK;
	}

	STDMETHOD(get_Count)(long *pVal)
	{
		*pVal = m_ColVector.size();

		return S_OK;
	}

	STDMETHOD(Item)(int Index, T ** pVal)
	{
		//Is the Index in range?
		//
		if ((Index < 0) || (Index >= m_ColVector.size()))
		{
			return E_INVALIDARG;
		}

		CComPtr<T> pT;
		pT = m_ColVector[Index];
		if (pT)
		{
			return pT->QueryInterface(__uuidof(T), (LPVOID *)pVal);
		}
		else
		{
			return E_INVALIDARG;
		}
	}

	STDMETHOD(get__NewEnum)(IUnknown **pVal)
	{
		m_ColIterator = m_ColVector.begin();
		
		T * pT = reinterpret_cast<T *>(this);
		return pT->QueryInterface(IID_IUnknown, (LPVOID *)pVal);
	}

	STDMETHOD(Remove)(int Index)
	{
		int SearchIndex = 0;
		
		//Locate the Iterator for the given Index
		//
		for (m_ColIterator = m_ColVector.begin();
			 m_ColIterator != m_ColVector.end();
			 m_ColIterator++, SearchIndex++)
		{
			if (Index == SearchIndex)
			{
				m_ColVector.erase(m_ColIterator);
				return S_OK;
			}
		}

		return E_INVALIDARG;
	}

	//IEnumVARIANT methods
	//
	STDMETHOD(Next)(ULONG celt, VARIANT *rgVar, ULONG *pCeltFetched)
	{
		if (pCeltFetched)
		{
			*pCeltFetched = 0;
		}

		if (celt > 1)
		{
			return E_INVALIDARG;
		}

		VariantInit(&rgVar[0]);

		if (m_ColIterator != m_ColVector.end())
		{
			rgVar[0].vt = VT_DISPATCH;
			T * pT = *m_ColIterator;
			pT->AddRef();
			rgVar[0].pdispVal = pT;

			if (pCeltFetched)
			{
				*pCeltFetched = 1;
			}

			m_ColIterator++;
		}
		else
		{
			rgVar[0].vt = VT_EMPTY;
			
			if (pCeltFetched)
			{
				*pCeltFetched = 0;
			}

			return S_FALSE;
		}

		return S_OK;
	}

	STDMETHOD(Skip)(ULONG celt)
	{
		m_ColIterator++;
		
		//Out of Range?
		//
		if (m_ColIterator == m_ColVector.end())
		{
			return E_INVALIDARG;
		}

		return S_OK;
	}

	STDMETHOD(Reset)()
	{
		//Reset Enumerator
		//
		m_ColIterator = m_ColVector.begin();

		return S_OK;
	}

	STDMETHOD(Clone)(IEnumVARIANT **ppEnum)
	{
		return E_NOTIMPL;
	}

// Vector to hold the Smart Pointers
private:
	COLLECTION_VECTOR		m_ColVector;
	COLLECTION_ITERATOR		m_ColIterator;
};


#endif //__COMCOLLECTION_H__